@@ -53,8 +53,9 @@ our $use_objectv = 1;
 use constant FBIT => 128;
 
 sub hobodecode {
+    return unless defined $_[0];
     open my $fh, "| $^X -Mblib=../Encoder -Mblib=../Decoder author_tools/hobodecoder.pl -e" or die $!;
-    print $fh @_;
+    print $fh $_[0];
     close $fh;
 }
 
@@ -110,7 +111,9 @@ sub dump_bless {
 
 sub short_string {
     die if length($_[0]) > SRL_MASK_SHORT_BINARY_LEN;
-    return chr(SRL_HDR_SHORT_BINARY_LOW + length($_[0])) . $_[0];
+    my $tag = SRL_HDR_SHORT_BINARY_LOW + length($_[0]);
+    $tag |= SRL_HDR_TRACK_FLAG if $_[1];
+    return pack("c a*",$tag,$_[0]);
 }
 
 sub integer {
@@ -141,9 +144,10 @@ sub varint {
 our $PROTO_VERSION;
 
 sub Header {
-    my $proto_version = shift || $PROTO_VERSION;
+    my $proto_version = shift || $PROTO_VERSION || SRL_PROTOCOL_VERSION;
     my $user_data_blob = shift;
-    my $hdr_base = SRL_MAGIC_STRING . chr($proto_version||SRL_PROTOCOL_VERSION);
+    my $mgc = $proto_version > 2 ? SRL_MAGIC_STRING_HIGHBIT : SRL_MAGIC_STRING;
+    my $hdr_base = $mgc . chr($proto_version);
     if (defined $user_data_blob) {
         return $hdr_base . varint(1 + length($user_data_blob)) . chr(1) . $user_data_blob;
     }
@@ -172,13 +176,17 @@ sub offseti {
 }
 
 sub debug_checks {
-    my ($data_ref, $encoded_ref, $decoded_ref) = @_;
-    if (defined $ENV{DEBUG_SEREAL}) {
-        note("Original data was: " . Data::Dumper::Dumper($$data_ref)) if defined $data_ref;
-        note("Encoded data is: " . (defined($$encoded_ref) ? $$encoded_ref : "<undef>")) if defined $encoded_ref;
-        note("Decoded data was: " . Data::Dumper::Dumper($$decoded_ref)) if defined $decoded_ref;
+    my ($data_ref, $encoded_ref, $decoded_ref, $debug) = @_;
+    if ($debug or defined $ENV{DEBUG_SEREAL}) {
+        note("Original data was: " . Data::Dumper::Dumper($$data_ref))
+            if defined $data_ref;
+        note("Encoded data is: " . (defined($$encoded_ref) ? Data::Dumper::qquote($$encoded_ref) : "<undef>"))
+            if defined $encoded_ref;
+        note("Decoded data was: " . Data::Dumper::Dumper($$decoded_ref))
+            if defined $decoded_ref;
     }
     if (defined $ENV{DEBUG_DUMP}) {
+        Dump($$data_ref)    if defined $data_ref;
         Dump($$encoded_ref) if defined $encoded_ref;
         Dump($$decoded_ref) if defined $decoded_ref;
     }
@@ -234,7 +242,7 @@ sub setup_tests {
                   my $d = array_head(3);
                   my $pos = offset($d);
                   my $tag = $opt->{aliased_dedupe_strings} ? SRL_HDR_ALIAS : SRL_HDR_COPY;
-                  $d .= short_string("foooo") . chr($tag) . varint($pos)
+                  $d .= short_string("foooo",$opt->{aliased_dedupe_strings} ? 1 : 0) . chr($tag) . varint($pos)
                         . chr($tag) . varint($pos);
                   return $d;
               }
@@ -258,7 +266,7 @@ sub setup_tests {
                   my $tag = $opt->{aliased_dedupe_strings} ? SRL_HDR_ALIAS : SRL_HDR_COPY;
                   my $d = array_head(2) . hash_head(2) . short_string("foooo");
                   my $pos = offset($d);
-                  $d .= short_string("foooo") . hash_head(2)
+                  $d .= short_string("foooo",$opt->{aliased_dedupe_strings} ? 1 : 0) . hash_head(2)
                         . short_string("foooo2")
                         . chr($tag) . varint($pos);
                   return $d;
@@ -547,6 +555,8 @@ sub have_encoder_and_decoder {
         return();
     };
     my $cmp_v = $need_class->VERSION;
+    $cmp_v =~ s/_//;
+    $cmp_v = sprintf("%.2f", int($cmp_v*100)/100);
     if (not defined $cmp_v or not exists $compat_versions{$cmp_v}) {
         note("Could not load correct version of $need_class for testing "
              ."(got: $cmp_v, needed any of ".join(", ", keys %compat_versions).")");
@@ -574,6 +584,12 @@ my $lots_of_9C = do {
 my $max_iv = ~0 >> 1;
 my $min_iv = do {use integer; -$max_iv-1}; # 2s complement assumption
 
+my $eng0e0= "0e0";
+my $eng0e1= "0e1";
+my $eng2= "1e3";
+
+my $sum= $eng0e0 + $eng0e1 + $eng2;
+
 our @ScalarRoundtripTests = (
     # name, structure
     ["undef", undef],
@@ -595,6 +611,7 @@ our @ScalarRoundtripTests = (
             0x7FFFFFFF, 0x80000000, 0x80000001, 0xFFFFFFFF, 0xDEADBEEF,
             # UV bounds
             $max_iv_p1, $max_uv_m1, $max_uv, $lots_of_9C,
+            $eng0e0, $eng0e1, $eng2,
         )
     ),
 
@@ -639,7 +656,7 @@ our @ScalarRoundtripTests = (
         'bop \'x\\x'    =>"x\x{100}"   , 'bing' =>   "x\x{100}",
         x=>'y',}, z => 'p' ,   }   ,
         i    =>  '1' ,}, l=>" \10", m=>"\10 ", n => " \10 ",
-        o => undef ,p=>undef,
+        o => undef ,p=>undef, q=>\undef, r=>\$eng0e0, u => \$eng0e1, w=>\$eng2
     }],
     ['var strings', [ "\$", "\@", "\%" ]],
     [ "quote keys", { "" => '"', "'" => "" }],
@@ -654,6 +671,8 @@ our @ScalarRoundtripTests = (
     [ "ref to undef", \undef],
     [ "negative big num", -4123456789],
     [ "positive big num", 4123456789],
+    [ "eng-ref", [\$eng0e0, \$eng0e1, \$eng2] ],
+    [ "undef", [\undef, \undef] ],
 );
 
 use Storable qw(dclone);
@@ -687,24 +706,25 @@ if (eval "use Array::RefElem (av_store hv_store); 1") {
 
 
 sub run_roundtrip_tests {
-    for my $proto_version (qw(2 1)) {
-        my $suffix = $proto_version == 1 ? "_v1" : "";
+    my ($name, $opts) = @_;
 
-        for my $opt (
-            ['plain',          {                  } ],
-            ['snappy',         { snappy         => 1 } ],
-            ['snappy_incr',    { snappy_incr    => 1 } ],
-            ['sort_keys',      { sort_keys      => 1 } ],
-            ['dedupe_strings', { dedupe_strings => 1 } ],
-        ) {
-            my ($name, $opts) = @$opt;
-            $name .= $suffix;
-            $opts->{use_protocol_v1} = 1 if $proto_version == 1;
-            $PROTO_VERSION= $proto_version;
-            setup_tests();
-            run_roundtrip_tests_internal($name, $opts);
-        }
+    my $proto_version;
+    if ( $0 =~ m![\\/]v(\d+)[\\/]!) {
+        $proto_version= $1;
+    } else {
+        die "Failed to detect version\n";
     }
+
+    my $suffix = "_v$proto_version";
+    if ($proto_version == 1) {
+        $opts->{use_protocol_v1} = 1;
+    }
+    else {
+        # v2 ignores this, but will output v2 by default
+        $opts->{protocol_version} = $proto_version;
+    }
+    setup_tests($proto_version);
+    run_roundtrip_tests_internal($name . $suffix, $opts);
 }
 
 sub run_roundtrip_tests_internal {
@@ -713,18 +733,21 @@ sub run_roundtrip_tests_internal {
     my $encoder = Sereal::Encoder->new($opt);
 
     foreach my $meth (
-                      ['functional',
-                        sub {Sereal::Encoder::encode_sereal(shift, $opt)},
-                        sub {Sereal::Decoder::decode_sereal(shift, $opt)}],
+                      ['functional simple',
+                        sub {Sereal::Encoder::encode_sereal($_[0], $opt)},
+                        sub {Sereal::Decoder::decode_sereal($_[0], $opt)}],
                       ['object-oriented',
-                        sub {$encoder->encode(shift)},
-                        sub {$decoder->decode(shift)}],
+                        sub {$encoder->encode($_[0])},
+                        sub {$decoder->decode($_[0])}],
+                      ['functional with object',
+                          sub {Sereal::Encoder::sereal_encode_with_object($encoder, $_[0])},
+                          sub {Sereal::Decoder::sereal_decode_with_object($decoder, $_[0])}],
                       ['header-body',
-                        sub {$encoder->encode(shift, 123456789)}, # header data is abitrary to stand out for debugging
-                        sub {$decoder->decode(shift)}],
+                        sub {$encoder->encode($_[0], 123456789)}, # header data is abitrary to stand out for debugging
+                        sub {$decoder->decode($_[0])}],
                       ['header-only',
-                        sub {$encoder->encode(987654321, shift)}, # body data is abitrary to stand out for debugging
-                        sub {$decoder->decode_only_header(shift)}],
+                        sub {$encoder->encode(987654321, $_[0])}, # body data is abitrary to stand out for debugging
+                        sub {$decoder->decode_only_header($_[0])}],
                       )
     {
         my ($mname, $enc, $dec) = @$meth;
@@ -732,6 +755,7 @@ sub run_roundtrip_tests_internal {
 
         foreach my $rt (@RoundtripTests) {
             my ($name, $data) = @$rt;
+
             my $encoded;
             eval {$encoded = $enc->($data); 1}
                 or do {
@@ -779,15 +803,15 @@ sub run_roundtrip_tests_internal {
                 or next;
             is_deeply($decoded, $data, "$name ($ename, $mname, decoded vs data)")
                 or do {
-                    debug_checks(\$data, \$encoded2, \$decoded2);
+                    debug_checks(\$data, undef, \$decoded, "debug");
                 };
             is_deeply($decoded2, $data, "$name ($ename, $mname, decoded2 vs data)")
                 or do {
-                    debug_checks(\$data, \$encoded2, \$decoded2);
+                    debug_checks(\$data, undef, \$decoded2, "debug");
                 };
-            is_deeply($decoded, $decoded2, "$name ($ename, $mname, decoded vs decoded2)")
+            is_deeply($decoded2, $decoded, "$name ($ename, $mname, decoded vs decoded2)")
                 or do {
-                    debug_checks(\$data, \$encoded2, \$decoded2);
+                    debug_checks(\$decoded, undef, \$decoded2, "debug");
                 };
 
             if (0) {
@@ -825,6 +849,7 @@ sub _write_file {
 }
 
 # For bootstrapping other language implementations' tests
+our $COMPRESS;
 sub write_test_files {
     my $dir = shift;
     require File::Path;
@@ -841,7 +866,10 @@ sub write_test_files {
         _write_file($make_name_file_name->($testno), $t->[2] . "\n");
     }
 
-    my $encoder = Sereal::Encoder->new;
+    my $encoder = Sereal::Encoder->new({
+        protocol_version => $PROTO_VERSION,
+        compress => $COMPRESS || Sereal::Encoder::SRL_UNCOMPRESSED(),
+    });
     foreach my $i (0..$#RoundtripTests) {
         my $testno = @BasicTests + $i + 1;
         my $t = $RoundtripTests[$i];
